@echo off & setlocal enabledelayedexpansion

cls


::	Version 2023-05-15 by NotNull
:: Changelog at the end of thsi script

:: More information:
:: https://www.voidtools.com/forum/viewtopic.php?t=6808


:: ===============================================
::
::				SETTINGS
::
:: ===============================================


::-----------------------------------------------
::	Location of Everything.exe (WITHOUT surrounding quotes)
::-----------------------------------------------
	rem 
	set EverythingEXE=C:\Tools\Everything\Everything.exe

 	rem set EverythingEXE=C:\Program Files\Everything 1.5a\Everything64.exe


::-----------------------------------------------
::	Icon to use in context menu (WITHOUT surrounding quotes)
::-----------------------------------------------

::	Default: Use IndexThis.ico in scriptfolder
::
::	Some examples:
::	set ICON=c:\path to\My Favorite.ico
::	set ICON=%SystemRoot%\system32\compstui.dll,-64040
::	set ICON=c:\program files\everything\everything.exe


::-----------------------------------------------
::	Number of seconds to keep showing the console
::	After Everything started
	set WAIT=1
::-----------------------------------------------


:: ===============================================
::
::				MAIN
::
:: ===============================================


::_______________________________________________
::
::				INIT
::_______________________________________________


::	$ has special meaning in Everything window title; escape it.	
	set TITLEPATH=%1
	set TITLEPATH=%TITLEPATH:$=$$%


	If not defined ICON set ICON=%~dpn0.ico
rem	set ICON="%ICON%"

	set "EXEversion="
	set "UseService="

	If not defined WAIT set WAIT=0

::_______________________________________________
::
::				ACTION!
::_______________________________________________
::
	If [%1]==[] goto :CONTEXT



 	call :Check4Service
	if defined UseService goto :BUILD


	call :Check4Process
	if defined EXEversion goto :BUILD

	call :Check4ScriptSetting
	If not defined EverythingEXE EXIT /B
	set EverythingEXE="%EverythingEXE%"

	echo	=======[ ScripSetting ]===========
	echo.
	echo.	No service
	echo.	Executable = [!EverythingEXE!]
	echo.





:: ===============================================
				:BUILD
:: ===============================================

::	Stop already running temp-index
 	start /wait "" %EverythingEXE% -nodb -instance TEMPINDEX  -config "%TEMP%\DeleteThis.ini" -quit

::	Create base-ini
	call :BASE-INI


::	Figure out if NTFS indexing is possible; otherwise Folder Indexing
::	Assume Folder Indexing until  proven otherwise.

	set INDEXMETHOD=FOLDER

	
::	Check this volume for filesystem and bootvolume

	for /f "usebackq" %%x in (`wmic.exe volume where DriveLetter^="%~d1" get FileSystem^,bootvolume /value`) Do set %%x>nul

	IF /i [%FileSystem%] neq [NTFS] (
		rem Keep using folder indexing
	) ELSE IF /i [%bootvolume%] neq [TRUE] (
		rem NTFS, but no service needed
		set INDEXMETHOD=NTFS
	) ELSE (
		rem NTFS and service needed
		If defined UseService set INDEXMETHOD=NTFS
	)



::	Show indexing method to use
	echo.
	echo.	Use %EXEversion%%MAJOR% %INDEXMETHOD% indexing for %1
	echo.
	echo.


::	Add filesystem-specific entries to the INI

	set ThisFolder=%1
	call :%INDEXMETHOD%-INI %ThisFolder%


::	INI complete; start Everything ...

 	start "" %EverythingEXE% -nodb -instance TEMPINDEX -config "%TEMP%\IndexThis.ini"
	
::	Keep showing console messages for 5 more seconds
::	TIMEOUT not available on older OS'es, so use PING
	

	ping -n %WAIT% -w 1000 localhost >nul

goto :EOF

(.. and we are done here ...)



::===============================================
::
::	SUBROUTINES
::
::===============================================
::

::-----------------------------------------------
				:Check4Service		
::-----------------------------------------------
::


::	Check if there is an Everything Service running to be used
::	If multiple: use highest version

	for /f "skip=1 usebackq delims=" %%x in (` wmic.exe service where "Name like 'Everything%%' AND State='Running'" get Name^,PathName `) Do (
		set RESULT=%%x
		set RESULT=!RESULT:"=/!
rem		echo RESULT=!RESULT!
		
	rem Workaround for workaround for WMIC bug ..
		for /f "usebackq tokens=1,2 delims=/" %%i in (`echo !RESULT!`) DO if exist "%%j" (

			call :GetLongVersion "%%j" ServiceVersion
			If "!ServiceVersion!" GTR "!EXEversion!" (
				set EverythingEXE="%%j"
				set "EXEversion=!ServiceVersion!"
			::	Strip leading and trailing spaces of servicename
				for /f "tokens=*" %%A in ("%%i") do set UseService=%%~nxA

rem				echo UseService = __!UseService!__
rem				echo ExePath = __!EverythingEXE!__
rem				set RESULT=
			)
			set ServiceVersion=
		)
	) 

	echo =======[ Service ]===========
	echo.
	echo	Service = [!UseService!]
	echo	Executable = [!EverythingEXE!]
	echo.	Version    = [!EXEversion!]
	echo.
	
	
goto :EOF


::-----------------------------------------------
				:Check4Process	
::-----------------------------------------------
::

::	Check if there is an Everything process running to be used
::	If multiple: use highest version



	for /f "usebackq skip=1 tokens=2 delims==" %%x in (`wmic.exe process where "Name like 'Everything%%'" get ExecutablePath /value`) DO (

		set thisEXE=%%x
	::	Workaround for WMIC Proceess CR bug
		set thisEXE="!thisEXE:~0,-1!"

		If exist !thisEXE! ( 
			call :GetLongVersion !thisEXE! ProcessVersion
		
			If "!ProcessVersion!" GTR "!EXEversion!" (

				set "EverythingEXE=!thisEXE!"
				set "EXEversion=!ProcessVersion!"
			)
		)
		set "thisEXE="
		set "ProcessVersion="
	)

	echo	=======[ Process ]===========
	echo.
	echo.	No service
	echo.	Executable = [!EverythingEXE!]
	echo.	Version    = [!EXEversion!]
	echo.
		
goto :EOF


::-----------------------------------------------
				:Check4ScriptSetting	
::-----------------------------------------------
::


	If not exist "%EverythingEXE%" (
		color 4F
		mode 70,15
		echo.
		echo.
		echo.	Path could not be found: 
		echo.	"%EverythingEXE%"
		echo.
		echo.	Check the "Set EverythingEXE=" setting in 
		echo.	the SETTINGS section of the script.
		echo.
		echo.
		set "EverythingEXE="
		pause
	)
goto :EOF



::-----------------------------------------------
				:BASE-INI		
::-----------------------------------------------

::	Create new base INI  (%TEMP%\IndexThis.ini)
	(echo [Everything]
	echo theme=2
	echo run_as_admin=0
	echo run_in_background=0
	echo show_in_taskbar=1
	echo show_tray_icon=0
	echo minimize_to_tray=0
	echo check_for_updates_on_startup=0
	echo auto_include_fixed_volumes=0
	echo auto_include_removable_volumes=0
	echo auto_include_fixed_refs_volumes=0
	echo auto_include_removable_refs_volumes=0
	echo hide_on_close=0
	echo window_title_format=$i %TITLEPATH%
	) > "%TEMP%\IndexThis.ini"

goto :EOF



::-----------------------------------------------
				:NTFS-INI
::-----------------------------------------------

:: Write NTFS settings to INI
::	Next to %1use parm directly due to CALL limitations ( ^^ )
	
:: Get VolumeID and convert \ to \\
	for /f "usebackq delims= " %%x in (`mountvol %~d1 /L`) Do set VOLUMEID=%%x
	set VOLUMEID=%VOLUMEID:\=\\%

	
	(
		echo ntfs_volume_guids="%VOLUMEID%"
	:: volume_paths MUST be in UPPERCASE
		echo ntfs_volume_paths="%~d1"
		echo ntfs_volume_roots=""
		echo ntfs_volume_includes=1
	) >> "%TEMP%\IndexThis.ini"


:: %ThisFolder% and PIPE separate because of 
::	possible () and special chars conflict

	set PIPE=\\.\PIPE\%UseService:Everything=Everything Service%
	echo service_pipe_name=%PIPE%>> "%TEMP%\IndexThis.ini"


::	1.4 and 1.5 use different INI format for ntfs_volume_include_onlys
	
	call :GetMajorVersion %EverythingEXE% MAJOR

	If [%MAJOR%]==[1.4] echo ntfs_volume_include_onlys=!ThisFolder:\=\\!>> "%TEMP%\IndexThis.ini"
	If [%MAJOR%] neq [1.4] for %%x in (%ThisFolder%) do echo ntfs_volume_include_onlys=%%~x>> "%TEMP%\IndexThis.ini"
	
goto :EOF



::-----------------------------------------------
				:FOLDER-INI
::-----------------------------------------------


:: Everything 1.4 and 1.5 use different INI settings for folder indexing
::	No extra spaces before redirection	

	call :GetMajorVersion %EverythingEXE% MAJOR

	If [%MAJOR%]==[1.4] echo folders=!ThisFolder:\=\\!>> "%TEMP%\IndexThis.ini"
	If [%MAJOR%] neq [1.4] for %%x in (%ThisFolder%) do echo folders=%%~x>> "%TEMP%\IndexThis.ini"

goto :EOF



::-----------------------------------------------
				:GetLongVersion		
::-----------------------------------------------
::
::				Gets the version of an executable
::				parm1 = path (quoted); 
::				parm2 = variable to put the result in
::________________________________________________
::
	set thisFile=%~1
	set thisFile=%thisFile:\=\\%

	for /f "skip=1 usebackq delims=" %%x in (`wmic.exe datafile where "name='%thisfile%'" get Version /value `) Do set "__%%x" 2>nul
	set %2=%__Version%
	
	set "thisFile="
	set "__Version="

goto :EOF



::-----------------------------------------------
				:GetMajorVersion		
::-----------------------------------------------
::
::				Gets the major version (1.4/1.5/.. of an executable
::				parm1 = path (quoted); 
::				parm2 = variable to put the result in
::___________________________________________________________
::
 	set thisFile=%~1
	set thisFile=%thisFile:\=\\%

	for /f "skip=1 usebackq delims=" %%x in (`wmic.exe datafile where "name='%thisfile%'" get Version /value `) Do set "__%%x" 2>nul
	set %2=%__Version:~0,3%
	
	set "thisFile="
	set "__Version="

goto :EOF



::-----------------------------------------------
				:CONTEXT
::-----------------------------------------------

	set INSTALLED=0

 	REG.exe query "HKCU\Software\Classes\Directory\Shell\IndexThis" 2>nul >nul && set "INSTALLED=1"

	If [%INSTALLED%]==[1] (
		REG DELETE HKCU\Software\Classes\Directory\Shell\IndexThis /f 2>nul >nul
		REG DELETE HKCU\Software\Classes\Drive\Shell\IndexThis /f 2>nul >nul
		
		mode 75,15
		color 1F
		echo.
		echo.
		echo.
		echo.		IndexThis context menu entry REMOVED
		echo.
		echo.		Re-run "%~nx0" to install again.
		echo.
		echo.
		pause
	EXIT /B
	)

::	This will only get executed when NOT instyalled


::	Check and escape special characters for the registry
	set REGFOLDER="%~f0"
	
	:: Foldernames with "^" and "%" are not allowed; EXIT
	set "NOGO="
	echo.
	echo %REGFOLDER%|findstr /i /C:"^" && set NOGO=1
	echo %REGFOLDER%|findstr /i /C:"%%" && set NOGO=1
	If defined NOGO (
		echo.
		echo.	Foldernames with "^" or "%%" are not allowed.
		echo.	Please install %~nx0 in a different folder
		echo.
		goto :EOF
	)
	
	set "REGFOLDER=%REGFOLDER:&=^^^&%"


::	Check Icon configuration
	For /f "delims=," %%x in ("%ICON%") do if not exist "%%x" set ERROR=Check ICON=%ICON%

	If Defined ERROR (
		echo.
		echo.  Icon misconfiguration
		echo.  Check the SETTINGS section of the script
		echo.
		echo.  ^( Script: %~f0 ^)
		echo.
		goto :EOF
	)
  
::-----------------------------------------------
::	Add contextmenu to registry for DIRECTORIES
::-----------------------------------------------

	::	Add command to execute
	REG.exe ADD HKCU\Software\Classes\Directory\Shell\IndexThis\Command /ve /d "%REGFOLDER:"=\"% \"%%1\"" -f 2>nul >nul

	::	Add icon to contexmenu
	REG.exe ADD HKCU\Software\Classes\Directory\Shell\IndexThis /v Icon /d "\"%ICON%\"" -f 2>nul >nul

	
::-----------------------------------------------
::	Add contextmenu to registry for DRIVES
::-----------------------------------------------

	::	Add command to execute
	REG ADD HKCU\Software\Classes\Drive\Shell\IndexThis\Command /ve /d "%REGFOLDER:"=\"% \"%%1\"" -f 2>nul >nul

	::	Add icon to contexmenu
	REG ADD HKCU\Software\Classes\Drive\Shell\IndexThis /v Icon /d "\"%ICON%\"" -f 2>nul >nul


	::	(Not checking if succeeded)
	
		mode 75,15
		color 2F
		echo.
		echo.
		echo.		IndexThis context menu entry INSTALLED
		echo.
		echo.		Re-run "%~nx0" to remove.
		echo.
		echo.
		pause
	EXIT /B






::===============================================
::				CHANGELOG
::===============================================


::	Updated version (2023-05-15)
:: - Workaround for WMIC bugs
::	- Fixed context menu icon (again)
:: - Configurable delay before closing console window
:: 
::===============================================
::
::	Rewritten version (2023-05-12)
::	- Added support for Everything 1.5 & 1.5 alpha
::	- Running services and processes are used to find Everything.exe
:: - Configured path in script is fallback
::	- Fixed ccontext menu icon
:: - More support for special characters in foldernames
:: 
::===============================================
::
::	Updated version (2019-12-17)
::	- Fixed an issue with $ in foldername to index.
::	  The $ wasn't shown correctly in Everything's windowtitle
::	- Fixed an issue when IndexThis was in a folder containing
::	  & in it's name.
::	- Installation in foldernames with "^"  is no longer allowed.
::	  The ^ is too "toxic"
::	- Fixed an issue with indexiong a folder with "()" in it's name.
::	- Use NTFS indexing for NTFS volumes that are not bootvolume
::	   without the service being needed.
::	- Added an option to use a different icon in the context menu
::
::===============================================
::
::	Updated version (2018-10-25)
::	- Use a different detection method of NTFS
::		filesystem that doesnt need admin rights
::	- Check if Everything service is running 
::		(needed for NTFS indexing)
::
::===============================================



